@@ -4,7 +4,7 @@
 		 2003 Tomaz Kac,
 		 2003 Philip Kendall
 
-   $Id: snap2tzx.c,v 1.12 2003/12/19 00:27:05 pak21 Exp $
+   $Id: snap2tzx.c 3711 2008-07-01 20:25:37Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,14 +16,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -34,7 +33,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/mman.h>
 #include <unistd.h>
 
 #include <libspectrum.h>
@@ -482,9 +480,9 @@ load_snap( libspectrum_snap **snap, const char *filename )
   int error;
   unsigned char *buffer; size_t length;
 
-  error = libspectrum_snap_alloc( snap ); if( error ) return error;
+  *snap = libspectrum_snap_alloc();
 
-  if( mmap_file( filename, &buffer, &length ) ) {
+  if( read_file( filename, &buffer, &length ) ) {
     libspectrum_snap_free( *snap );
     return 1;
   }
@@ -492,16 +490,11 @@ load_snap( libspectrum_snap **snap, const char *filename )
   error = libspectrum_snap_read( *snap, buffer, length, LIBSPECTRUM_ID_UNKNOWN,
 				 filename );
   if( error ) {
-    libspectrum_snap_free( *snap ); munmap( buffer, length );
+    libspectrum_snap_free( *snap ); free( buffer );
     return error;
   }
 
-  if( munmap( buffer, length ) == -1 ) {
-    fprintf( stderr, "%s: couldn't munmap '%s': %s\n", progname, filename,
-	     strerror( errno ) );
-    libspectrum_snap_free( *snap );
-    return 1;
-  }
+  free( buffer );
 
   return 0;
 }
@@ -877,18 +870,15 @@ add_rom_block( libspectrum_tape *tape, const libspectrum_byte flag,
 {
   libspectrum_byte *buffer;
   libspectrum_tape_block *block;
-  int error;
-
   buffer = malloc( ( length + 2 ) * sizeof( libspectrum_byte ) );
   if( !buffer ) {
     print_error( "out of memory at %s:%d", __FILE__, __LINE__ );
     return 1;
   }
 
-  error = libspectrum_tape_block_alloc( &block, LIBSPECTRUM_TAPE_BLOCK_ROM );
-  if( error ) { free( buffer ); return error; }
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_ROM );
 
-  libspectrum_tape_block_set_pause( block, 0 );
+  libspectrum_tape_block_set_pause( block, 100 );
   libspectrum_tape_block_set_data_length( block, length + 2 );
   libspectrum_tape_block_set_data( block, buffer );
 
@@ -896,11 +886,7 @@ add_rom_block( libspectrum_tape *tape, const libspectrum_byte flag,
   memcpy( buffer + 1, data, length );
   buffer[ length + 1 ] = calc_checksum( buffer, length + 1 );
 
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) {
-    libspectrum_tape_block_free( block );
-    return error;
-  }
+  libspectrum_tape_append_block( tape, block );
 
   return 0;
 }
@@ -1090,7 +1076,6 @@ add_loader_block( libspectrum_tape *tape, libspectrum_byte **loader,
 		  libspectrum_snap *snap, const settings_t *settings )
 {
   libspectrum_tape_block *block;
-  libspectrum_error error;
   libspectrum_byte *basic;
   size_t length;
   int error2;
@@ -1103,10 +1088,9 @@ add_loader_block( libspectrum_tape *tape, libspectrum_byte **loader,
     return 1;
   }
 
-  error = libspectrum_tape_block_alloc( &block, LIBSPECTRUM_TAPE_BLOCK_ROM );
-  if( error ) { free( *loader ); return error; }
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_ROM );
 
-  libspectrum_tape_block_set_pause( block, 0 );
+  libspectrum_tape_block_set_pause( block, 100 );
   libspectrum_tape_block_set_data_length( block, length );
   libspectrum_tape_block_set_data( block, (*loader) );
 
@@ -1125,14 +1109,13 @@ add_loader_block( libspectrum_tape *tape, libspectrum_byte **loader,
 
   /* Put the loader into the buffer */
   error2 = create_loader( basic + basic_length, snap, settings );
-  if( error2 ) { libspectrum_tape_block_free( block ); return error; }
+  if( error2 ) { libspectrum_tape_block_free( block ); return error2; }
   
   /* We don't calculate the checksum yet as we need to do it after
      we've put the page lengths in the data */
 
   /* But put the block into the tape anyway */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return 0;
 }
@@ -1154,7 +1137,7 @@ create_turbo_header( libspectrum_tape_block *block, int speed )
   libspectrum_tape_block_set_bit0_length( block, ptr->bit_length );
   libspectrum_tape_block_set_bit1_length( block, ptr->bit_length * 2 );
   libspectrum_tape_block_set_bits_in_last_byte( block, 8 );
-  libspectrum_tape_block_set_pause( block, 0 );
+  libspectrum_tape_block_set_pause( block, 100 );
 
   return 0;
 }
@@ -1201,8 +1184,7 @@ add_page( libspectrum_tape *tape, libspectrum_snap *snap, int page,
   size_t page_length, compressed_length, reverse_offset;
   libspectrum_byte table_byte, *buffer;
 
-  error = libspectrum_tape_block_alloc( &block, LIBSPECTRUM_TAPE_BLOCK_TURBO );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_TURBO );
 
   error = create_turbo_header( block, settings->speed ); if( error ) {
     libspectrum_tape_block_free( block );
@@ -1341,8 +1323,7 @@ add_page( libspectrum_tape *tape, libspectrum_snap *snap, int page,
 
   (*loader_table_entry)++;
 
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return 0;
 }
@@ -1461,7 +1442,8 @@ write_tape( libspectrum_tape *tape, const char *filename )
 
   length = 0;
 
-  error = libspectrum_tzx_write( &buffer, &length, tape );
+  error = libspectrum_tape_write( &buffer, &length, tape,
+                                  LIBSPECTRUM_ID_TAPE_TZX );
   if( error ) return error;
 
   f = fopen( filename, "wb" );
@@ -1497,13 +1479,11 @@ write_tape( libspectrum_tape *tape, const char *filename )
 static int
 convert_snap( libspectrum_snap *snap, const settings_t *settings )
 {
-  libspectrum_tape *tape;
+  libspectrum_tape *tape = libspectrum_tape_alloc();
   int error;
 
   print_verbose( "\nCreating TZX file:\n" );
 
-  error = libspectrum_tape_alloc( &tape ); if( error ) return error;
-
   error = create_main_header( tape, settings->loader_name );
   if( error ) { libspectrum_tape_free( tape ); return error; }
 
