@@ -1,7 +1,7 @@
 /* snapconv.c: Convert snapshot formats
-   Copyright (c) 2003 Philip Kendall
+   Copyright (c) 2003-2005 Philip Kendall
 
-   $Id: snapconv.c,v 1.4 2003/10/20 14:39:22 pak21 Exp $
+   $Id: snapconv.c 3702 2008-06-30 20:34:26Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -31,10 +30,10 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <sys/mman.h>
 
 #include <libspectrum.h>
 
+#include "compat.h"
 #include "utils.h"
 
 char *progname;
@@ -47,47 +46,59 @@ main( int argc, char **argv )
   unsigned char *buffer; size_t length;
   libspectrum_creator *creator;
   int flags;
+  int compress = 0;
   FILE *f;
 
   int error;
+  int c;
 
   progname = argv[0];
 
-  if( argc < 3 ) {
-    fprintf( stderr, "%s: usage: %s <infile> <outfile>\n", progname,
+  while( ( c = getopt( argc, argv, "cn" ) ) != -1 ) {
+
+    switch( c ) {
+
+    case 'c': compress = LIBSPECTRUM_FLAG_SNAPSHOT_ALWAYS_COMPRESS;
+      break;
+
+    case 'n': compress = LIBSPECTRUM_FLAG_SNAPSHOT_NO_COMPRESSION;
+      break;
+
+    }
+  }
+  argc -= optind;
+  argv += optind;
+
+  if( argc < 2 ) {
+    fprintf( stderr, "%s: usage: %s [-c] [-n] <infile> <outfile>\n", progname,
 	     progname );
     return 1;
   }
 
   error = init_libspectrum(); if( error ) return error;
 
-  error = libspectrum_snap_alloc( &snap ); if( error ) return error;
+  snap = libspectrum_snap_alloc();
 
-  if( mmap_file( argv[1], &buffer, &length ) ) {
+  if( read_file( argv[0], &buffer, &length ) ) {
     libspectrum_snap_free( snap );
     return 1;
   }
 
   error = libspectrum_snap_read( snap, buffer, length, LIBSPECTRUM_ID_UNKNOWN,
-				 argv[1] );
+				 argv[0] );
   if( error ) {
-    libspectrum_snap_free( snap ); munmap( buffer, length );
+    libspectrum_snap_free( snap ); free( buffer );
     return error;
   }
 
-  if( munmap( buffer, length ) == -1 ) {
-    fprintf( stderr, "%s: couldn't munmap '%s': %s\n", progname, argv[1],
-	     strerror( errno ) );
-    libspectrum_snap_free( snap );
-    return 1;
-  }
+  free( buffer );
 
-  error = libspectrum_identify_file_with_class( &type, &class, argv[2], NULL,
-						0 );
+  error = libspectrum_identify_file_with_class( &type, &class, argv[1], NULL,
+                                                0 );
   if( error ) { libspectrum_snap_free( snap ); return error; }
 
   if( class != LIBSPECTRUM_CLASS_SNAPSHOT ) {
-    fprintf( stderr, "%s: '%s' is not a snapshot file\n", progname, argv[2] );
+    fprintf( stderr, "%s: '%s' is not a snapshot file\n", progname, argv[1] );
     libspectrum_snap_free( snap );
     return 1;
   }
@@ -97,7 +108,7 @@ main( int argc, char **argv )
 
   length = 0;
   error = libspectrum_snap_write( &buffer, &length, &flags, snap, type,
-				  creator, 0 );
+                                  creator, compress );
   if( error ) {
     libspectrum_creator_free( creator ); libspectrum_snap_free( snap );
     return error;
@@ -118,16 +129,16 @@ main( int argc, char **argv )
   error = libspectrum_snap_free( snap );
   if( error ) { free( buffer ); return error; }
 
-  f = fopen( argv[2], "w" );
+  f = fopen( argv[1], "wb" );
   if( !f ) {
-    fprintf( stderr, "%s: couldn't open '%s': %s\n", progname, argv[2],
+    fprintf( stderr, "%s: couldn't open '%s': %s\n", progname, argv[1],
 	     strerror( errno ) );
     free( buffer );
     return 1;
   }
     
   if( fwrite( buffer, 1, length, f ) != length ) {
-    fprintf( stderr, "%s: error writing to '%s'\n", progname, argv[2] );
+    fprintf( stderr, "%s: error writing to '%s'\n", progname, argv[1] );
     free( buffer );
     return 1;
   }
