@@ -1,7 +1,7 @@
 /* utils.c: useful utility functions
-   Copyright (c) 2002-2003 Philip Kendall
+   Copyright (c) 2002-2007 Philip Kendall
 
-   $Id: utils.c,v 1.6 2003/12/03 13:18:39 pak21 Exp $
+   $Id: utils.c 3702 2008-06-30 20:34:26Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -33,11 +32,14 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include <sys/mman.h>
+
+#ifndef WIN32
 #include <sys/utsname.h>
+#endif
 
 #include <libspectrum.h>
 
+#include "compat.h"
 #include "utils.h"
 
 extern char *progname;
@@ -64,19 +66,22 @@ get_creator( libspectrum_creator **creator, const char *program )
 {
   char *custom;
   int version[4] = { 0, 0, 0, 0 };
-  struct utsname buf;
-  libspectrum_error error; int sys_error;
+  libspectrum_error error;
   size_t i;
 
+#ifndef WIN32
+  struct utsname buf;
+  int sys_error;
+
   sys_error = uname( &buf );
   if( sys_error == -1 ) {
     fprintf( stderr, "%s: error getting system information: %s\n", progname,
 	     strerror( errno ) );
     return 1;
   }
+#endif
 
-  error = libspectrum_creator_alloc( creator );
-  if( error ) return error;
+  *creator = libspectrum_creator_alloc();
 
   error = libspectrum_creator_set_program( *creator, program );
   if( error ) { libspectrum_creator_free( *creator ); return error; }
@@ -101,12 +106,18 @@ get_creator( libspectrum_creator **creator, const char *program )
     return 1;
   }
 
+#ifdef WIN32
+  snprintf( custom, 256, "libspectrum: %s\nsystem: windows\n",
+	    libspectrum_version() );
+#else
   snprintf( custom, 256, "libspectrum: %s\nuname: %s %s %s\n",
 	    libspectrum_version(),
 	    buf.sysname, buf.machine, buf.release );
+#endif
 
   error = libspectrum_creator_set_custom( *creator,
-					  custom, strlen( custom ) );
+					  (libspectrum_byte*)custom,
+					  strlen( custom ) );
   if( error ) {
     free( custom ); libspectrum_creator_free( *creator );
     return error;
@@ -116,11 +127,12 @@ get_creator( libspectrum_creator **creator, const char *program )
 }
 
 int
-mmap_file( const char *filename, unsigned char **buffer, size_t *length )
+read_file( const char *filename, unsigned char **buffer, size_t *length )
 {
   int fd; struct stat file_info;
+  ssize_t bytes;
   
-  if( ( fd = open( filename, O_RDONLY ) ) == -1 ) {
+  if( ( fd = open( filename, O_RDONLY | O_BINARY ) ) == -1 ) {
     fprintf( stderr, "%s: couldn't open `%s': %s\n", progname, filename,
 	     strerror( errno ) );
     return 1;
@@ -133,20 +145,35 @@ mmap_file( const char *filename, unsigned char **buffer, size_t *length )
     return 1;
   }
 
-  (*length) = file_info.st_size;
+  *length = file_info.st_size;
 
-  (*buffer) = mmap( 0, *length, PROT_READ, MAP_SHARED, fd, 0 );
-  if( (*buffer) == (void*)-1 ) {
-    fprintf( stderr, "%s: couldn't mmap `%s': %s\n", progname, filename,
-	     strerror( errno ) );
+  *buffer = malloc( *length );
+  if( !*buffer ) {
+    fprintf( stderr, "%s: out of memory allocating %lu bytes\n", progname,
+             (unsigned long)*length );
     close(fd);
     return 1;
   }
 
+  bytes = read( fd, *buffer, *length );
+  if( bytes == -1 ) {
+    fprintf( stderr, "%s: error reading from `%s': %s\n", progname, filename,
+             strerror( errno ) );
+    free( *buffer );
+    close(fd);
+    return 1;
+  } else if( bytes < *length ) {
+    fprintf( stderr, "%s: could read only %lu out of %lu bytes from `%s'\n",
+             progname, (unsigned long)bytes, (unsigned long)*length, filename );
+    free( *buffer );
+    close(fd);
+    return 1;
+  }
+    
   if( close(fd) ) {
     fprintf( stderr, "%s: couldn't close `%s': %s\n", progname, filename,
 	     strerror( errno ) );
-    munmap( *buffer, *length );
+    free( *buffer );
     return 1;
   }
   
